package com.mt.wms.basic.controller;

import com.mt.wms.basic.params.AlarmBaseParam;
import com.mt.wms.basic.params.AlarmBaseQueryParam;
import com.mt.wms.basic.service.AlarmBaseService;
import com.mt.wms.basic.vo.AlarmBaseVo;
import com.mt.wms.core.base.BaseController;
import com.mt.wms.core.constants.CommonConstant;
import com.mt.wms.core.params.IdParam;
import com.mt.wms.core.utils.IDGenerator;
import com.mt.wms.core.validator.groups.AddGroup;
import com.mt.wms.core.validator.groups.PageGroup;
import com.mt.wms.core.validator.groups.UpdateGroup;
import com.mt.wms.core.vo.IdVo;
import com.mt.wms.core.vo.PageVo;
import com.mt.wms.core.vo.R;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.groups.Default;
import java.util.List;

/**
 * @Author: liguanghao
 * @Date: 2021/10/9 11:09
 * @Version 1.0
 */
@RestController
@RequestMapping(CommonConstant.API_MODULE_BASE + "alarmBase")
@Slf4j
@Api(value = "报警基础信息管理", tags = "报警基础信息管理", hidden = false)
public class AlarmBaseController extends BaseController {

    @Autowired
    private AlarmBaseService alarmBaseService;

    @PostMapping(value = "get")
    @ApiOperation(value = "获取报警基础信息")
    private R<AlarmBaseVo> get(@Validated @RequestBody IdParam idParam) {
        return alarmBaseService.get(idParam);
    }

    @PostMapping(value = "list")
    @ApiOperation(value = "获取报警基础信息列表")
    private R<List<AlarmBaseVo>> list(@Validated({Default.class}) @RequestBody AlarmBaseQueryParam alarmBaseQueryParam) {
        return alarmBaseService.list(alarmBaseQueryParam);
    }

    @PostMapping(value = "page")
    @ApiOperation(value = "获取分页报警基础信息")
    private R<PageVo<AlarmBaseVo>> page(@Validated({PageGroup.class, Default.class}) @RequestBody AlarmBaseQueryParam alarmBaseQueryParam) {
        return alarmBaseService.page(alarmBaseQueryParam);
    }

    @PostMapping(value = "add")
    @ApiOperation(value = "新增")
    private R<IdVo> add(@Validated({AddGroup.class, Default.class}) @RequestBody AlarmBaseParam alarmBaseParam) {
        return alarmBaseService.add(alarmBaseParam);
    }

    @PostMapping(value = "update")
    @ApiOperation(value = "更新")
    private R<IdVo> update(@Validated({UpdateGroup.class, Default.class}) @RequestBody AlarmBaseParam alarmBaseParam) {
        return alarmBaseService.update(alarmBaseParam);
    }

    @PostMapping(value = "delete")
    @ApiOperation(value = "删除报警基础信息")
    private R<IdVo> delete(@Validated @RequestBody IdParam idParam) {
        return alarmBaseService.delete(idParam);
    }

    @PostMapping(value = "codeGenerator")
    @ApiOperation(value = "编码生成")
    private R<String> codeGenerator() {
        return successful(IDGenerator.gen("BJ", "yyyyMMddHHmm", 2, "ALARMBASE_CODE"));
    }
}
