package com.mt.wms.empty.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.mt.wms.basic.service.KilnInfoService;
import com.mt.wms.basic.vo.KilnInfoVo;
import com.mt.wms.core.base.BaseController;
import com.mt.wms.core.constants.CommonConstant;
import com.mt.wms.core.dal.entity.AutoExeTask;
import com.mt.wms.core.dal.entity.CurrTask;
import com.mt.wms.core.dal.service.AutoExeTaskServiceBiz;
import com.mt.wms.core.dal.service.CurrTaskServiceBiz;
import com.mt.wms.core.params.BasePageParam;
import com.mt.wms.core.params.IdParam;
import com.mt.wms.core.utils.CodeGeneratorHelper;
import com.mt.wms.core.utils.HttpClient;
import com.mt.wms.core.vo.IdVo;
import com.mt.wms.core.vo.PageVo;
import com.mt.wms.core.vo.R;
import com.mt.wms.empty.params.*;
import com.mt.wms.empty.service.CurrTaskDetService;
import com.mt.wms.empty.service.CurrTaskService;
import com.mt.wms.empty.service.OrderInfoService;
import com.mt.wms.empty.task.TaskDistanceUtils;
import com.mt.wms.empty.vo.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.io.IOException;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;

/**
 * @author xcc
 * @date 2021年11月8日
 * @since 1.0
 */
@RestController
@RequestMapping(CommonConstant.API_MODULE_BASE + "currTask")
@Slf4j
@Api(value = "任务管理", tags = "当前任务管理", hidden = false)
public class CurrTaskController extends BaseController {
    @Autowired
    private CurrTaskService currTaskService;
    @Autowired
    private CurrTaskDetService currTaskDetService;
    @Autowired
    private OrderInfoService orderInfoService;
    @Autowired
    private ApmsController apmsControl;
    @Autowired
    private KilnInfoService kilnInfoService;
    @Autowired
    private CurrTaskServiceBiz currTaskServiceBiz;
    @Autowired
    private AutoExeTaskServiceBiz autoExeTaskServiceBiz;
    @Autowired
    private TaskDistanceUtils taskDistanceUtils;


    @PostMapping(value = "createProcessTask")
    @ApiOperation(value = "订单加工-创建一个加工任务至任务队列")
    public R<String> createProcessTask(@Validated @RequestBody TaskCreateParam param) {
        //验证标识卡号正确无误。
        orderInfoService.verifyTaskInfoByIdenCardNum(param.getDetParams());
        //验证炉子编码信息在apms正确无误
        R<KilnInfoVo> kilnInfoVoR = kilnInfoService.get(IdParam.builder().id(param.getKilnId()).build());
        String kilnCode = kilnInfoVoR.getData().getCode();
        R<ApmsStoveVo> bm = apmsControl.getStoveCodeByWorkShopCode("BM");
        ArrayList<Map<String, String>> stoveCodes = bm.getData().getStoveCodes();
        //验证apms数据中存在这个炉子
        boolean verifyCodes = false;
        ArrayList<String> codeStr = new ArrayList<>();
        stoveCodes.forEach(kilnMap -> {
            codeStr.add(kilnMap.get("code"));
        });
        for (String s : codeStr)
        {
            if (s.equals(kilnCode))
            {
                verifyCodes = true;
                break;
            }
        }
        if (!verifyCodes)
        {
            return failed("创建失败，APMS中验证炉号" + kilnCode + "失败！");
        }

        //通知apms创建一个生产单
        //ApmsCreateProcessSheet apmsCreateSheet = new ApmsCreateProcessSheet();
        //apmsCreateSheet.setWorkShopCode("BM");
        //apmsCreateSheet.setStartTime(LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));
        //apmsCreateSheet.setTechCode(param.getCraftCode());
        //fixme 暂时没有正式user，使用测试账户
        //apmsCreateSheet.setStartUser("QJJP03");
        ArrayList<CreateItem> createItems = new ArrayList<>();
        param.getDetParams().forEach(e -> {
            CreateItem createItem = new CreateItem();
            createItem.setItemCode(e.getIdenCardNum());
            createItem.setQuantity(e.getQuantity().intValue());
            createItem.setWeight(e.getWeight().doubleValue());
            createItems.add(createItem);
        });
        //apmsCreateSheet.setStoveCode(kilnCode);
        //apmsCreateSheet.setItems(createItems);
        //R<ApmsCreateProcessSheetVo> processSheet = apmsControl.createProcessSheet(apmsCreateSheet);

        //if (processSheet.getData().getSuccess())
        //{
            //apms创建成功后写入数据库
            param.setAutoTaskType(0);
            R<IdVo> mainTask = currTaskService.createProcessTask(param);
            currTaskDetService.createProcessTaskDet(param.getDetParams(), mainTask.getData().getId());
            //String sheetNo = processSheet.getData().getSheetNo();
            //将返回的生产单号写入到主任务中。
            //CurrTask currTaskById = currTaskService.getCurrTaskById(mainTask.getData().getId());
            //currTaskById.setSheetNo(sheetNo);
            //currTaskService.updateCurrTaskById(currTaskById);
            //生成详细任务中的检验结果数据在apms推送生产单结果时创建
            return successful(0,"创建成功，任务添加至等待执行队列。详情请在订单加工管理下当前任务列表查看","");
        //}
        //else
        //{
        //    return failed("创建失败，APMS报错：" + processSheet.getData().getMsg());
        //}
    }

    @PostMapping(value = "createAutoTask")
    @ApiOperation(value = "订单加工-创建多个个分步骤加工任务至任务队列")
    public R<String> createAutoTask(@Validated @RequestBody AutoCurrTaskParam autoCurrTaskParam){
        List<TaskCreateParam> detParams = autoCurrTaskParam.getTaskCreateParamList();
        if (detParams.size()==0){
            return failed("请添加至少一个标识卡！");
        }
        Boolean apmsCreateProcessSheet=false;
        String sheetNo=null;
        int i=1;
        R<ApmsCreateProcessSheetVo> processSheet=null;
        //自动任务循环添加多条currTask
        AutoExeTask autoExeTask=new AutoExeTask();
        for (TaskCreateParam param:detParams
             ) {
            //验证标识卡号正确无误。
            orderInfoService.verifyTaskInfoByIdenCardNum(param.getDetParams());
            //验证炉子编码信息在apms正确无误
            R<KilnInfoVo> kilnInfoVoR = kilnInfoService.get(IdParam.builder().id(param.getKilnId()).build());
            String kilnCode = kilnInfoVoR.getData().getCode();
            R<ApmsStoveVo> bm = apmsControl.getStoveCodeByWorkShopCode("BM");
            ArrayList<Map<String, String>> stoveCodes = bm.getData().getStoveCodes();
            //验证apms数据中存在这个炉子
            boolean verifyCodes = false;
            ArrayList<String> codeStr = new ArrayList<>();
            stoveCodes.forEach(kilnMap -> {
                codeStr.add(kilnMap.get("code"));
            });
            for (String s : codeStr)
            {
                if (s.equals(kilnCode))
                {
                    verifyCodes = true;
                    break;
                }
            }
            if (!verifyCodes)
            {
                return failed("创建失败，APMS中验证炉号" + kilnCode + "失败！");
            }
/*
            if (i==1){
                //通知apms创建一个生产单
                ApmsCreateProcessSheet apmsCreateSheet = new ApmsCreateProcessSheet();
                apmsCreateSheet.setWorkShopCode("BM");
                apmsCreateSheet.setStartTime(LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));
                apmsCreateSheet.setTechCode(param.getCraftCode());
                //fixme 暂时没有正式user，使用测试账户
                apmsCreateSheet.setStartUser("QJJP03");
                ArrayList<CreateItem> createItems = new ArrayList<>();
                param.getDetParams().forEach(e -> {
                    CreateItem createItem = new CreateItem();
                    createItem.setItemCode(e.getIdenCardNum());
                    createItem.setQuantity(e.getQuantity().intValue());
                    createItem.setWeight(e.getWeight().doubleValue());
                    createItems.add(createItem);
                });
                apmsCreateSheet.setStoveCode(kilnCode);
                apmsCreateSheet.setItems(createItems);
                 processSheet = apmsControl.createProcessSheet(apmsCreateSheet);
                apmsCreateProcessSheet=processSheet.getData().getSuccess();
                sheetNo=processSheet.getData().getSheetNo();
            }
*/

            //if (apmsCreateProcessSheet)
            //{
                //apms创建成功后写入数据库
                R<IdVo> mainTask = currTaskService.createProcessTask(param);
                currTaskDetService.createProcessTaskDet(param.getDetParams(), mainTask.getData().getId());
                CurrTask currTask = currTaskServiceBiz.getById(mainTask.getData().getId());
                //工艺步骤中第一步
                if (param.getAutoTaskStep()==1){
                    autoExeTask.setInterCode(CodeGeneratorHelper.getAutoTaskCode());
                    autoExeTask.setProcessFlowType(param.getAutoTaskType());
                    autoExeTask.setFTaskId(currTask.getId());
                    autoExeTask.setFDetTaskCode(currTask.getTaskCode());
                    autoExeTask.setFEquipmentId(currTask.getKilnId());
                    autoExeTask.setFEquipmentName(currTask.getKilnName());
                    autoExeTask.setFCraftCodeId(currTask.getCraftCodeId());
                    autoExeTask.setFPlcValue(currTask.getPlcValue());
                    setCommonField(autoExeTask);
                    autoExeTaskServiceBiz.save(autoExeTask);
                }
                if (param.getAutoTaskStep()==2){
                    autoExeTask.setTTaskId(currTask.getId());
                    autoExeTask.setTDetTaskCode(currTask.getTaskCode());
                    autoExeTask.setTEquipmentId(currTask.getKilnId());
                    autoExeTask.setTEquipmentName(currTask.getKilnName());
                    autoExeTask.setTCraftCodeId(currTask.getCraftCodeId());
                    autoExeTask.setTPlcValue(currTask.getPlcValue());
                    setUpdateCommonField(autoExeTask);
                    autoExeTaskServiceBiz.updateById(autoExeTask);
                }
                if (param.getAutoTaskStep()==3){
                    autoExeTask.setThTaskId(currTask.getId());
                    autoExeTask.setThDetTaskCode(currTask.getTaskCode());
                    autoExeTask.setThEquipmentId(currTask.getKilnId());
                    autoExeTask.setThEquipmentName(currTask.getKilnName());
                    autoExeTask.setThCraftCodeId(currTask.getCraftCodeId());
                    autoExeTask.setThPlcValue(currTask.getPlcValue());
                    setUpdateCommonField(autoExeTask);
                    autoExeTaskServiceBiz.updateById(autoExeTask);
                }
                //将返回的生产单号写入到主任务中。
                CurrTask currTaskById = currTaskService.getCurrTaskById(mainTask.getData().getId());
                //currTaskById.setSheetNo(sheetNo);
                currTaskById.setAutoExeTaskId(autoExeTask.getId());
                currTaskService.updateCurrTaskById(currTaskById);
                //生成详细任务中的检验结果数据在apms推送生产单结果时创建
                //return successful("创建成功，任务添加至等待执行队列。");

            //}
            //else
            //{
            //    return failed("创建失败，APMS报错：" + processSheet.getData().getMsg());
            //}
            //i++;
        }
        return successful(0,"操作成功,任务加入待执行队列,详情请在订单加工管理下当前任务列表查看！","");
    }

    @PostMapping(value = "mainCurrentTaskNow")
    @ApiOperation(value = "首页-获取当前执行中的任务")
    public R<PageVo<CurrTaskMainQueryVo>> mainCurrentTaskNow(@Validated @RequestBody BasePageParam param) {
        return currTaskService.currentTaskMainPage(param);
    }

    @PostMapping(value = "currentTaskNow")
    @ApiOperation(value = "获取当前执行的任务")
    public R<PageVo<CurrTaskQueryVo>> currentTask(@Validated @RequestBody CurrTaskQueryParam param) {
        return currTaskService.currentTaskPage(param);
    }

    @PostMapping(value = "currentTaskInfo")
    @ApiOperation(value = "获取当前任务列表以及详情")
    public R<List<CurrTaskInfoVo>> currentTaskInfo(@Validated @RequestBody CurrTaskQueryParam param) {
        return currTaskService.currentTaskInfoPage(param);
    }
    @PostMapping(value = "updateCurrTaskStatus")
    @ApiOperation(value = "更新任务状态")
    public R<String> updateCurrTaskStatus(@Validated @RequestBody CurrTaskParam param){
        return currTaskService.updateCurrTaskStatus(param);
    }
    @PostMapping(value = "completeTask")
    @ApiOperation(value = "执行任务完成（仅wms，不通知apms）")
    public R<String> completeTask(@Validated @RequestBody IdParam param) {
        return currTaskService.completeTask(param);
    }

    @PostMapping(value = "deleteBySheetNo")
    @ApiOperation(value = "人工通过生产单编号删除未正式开始的生产单(通知APMS)")
    public R<String> deleteBySheetNo(@Validated @RequestBody CurrTaskQueryParam param) {
        return currTaskService.deleteBySheetNo(param.getSheetNo());
    }

    @PostMapping(value = "deleteById")
    @ApiOperation(value = "人工id删除未正式开始的生产单")
    public R<String> deleteById(@Validated @RequestBody IdParam param) {
        return currTaskService.deleteById(param);
    }

    @PostMapping(value = "top")
    @ApiOperation(value = "置顶该任务")
    public R<String> top(@Validated @RequestBody IdParam param) {
        return currTaskService.top(param);
    }
    @PostMapping(value = "runTask")
    @ApiOperation(value = "执行任务")
    public R<String> runTask(@Validated @RequestBody IdParam param) throws InterruptedException, IOException {
        //点击执行按钮会判断当前任务是否为自动任务，如果是自动任务会去自动任务查一下该自动任务的上一步是否已完成（仅非一步时）
        return currTaskService.runTask(param.getId());
    }
    @PostMapping(value = "testRunTask")
    @ApiOperation(value = "执行任务")
    public R<String> testRunTask(@RequestBody RGVTask rgvTask){
        sendTaskToRgv(rgvTask.getTaskType(),rgvTask.getVehicleId(),rgvTask.getTaskNo(),rgvTask.getPickRow(),rgvTask.getSendRow(),rgvTask.getProcessNumber());
        return null;
    }
    @PostMapping(value = "testRgv")
    @ApiOperation(value = "测试rgv避让逻辑")
    private String testRgv(@RequestBody TestRgv testRgv) throws IOException {
        Long aLong = taskDistanceUtils.chooseVehicle(testRgv.getStartPoint(), testRgv.getEndPoint());
        Boolean aBoolean = taskDistanceUtils.conflictForVehicle(testRgv.getVehicleId(), testRgv.getStartPoint(), testRgv.getEndPoint(), testRgv.getOtherVehicle());
        return String.valueOf(aLong)+String.valueOf(aBoolean);
    }
    private Integer sendTaskToRgv(Integer taskType,Long vehicleId, Long currTaskId, Long startPoint, Long endPoint, Integer processNumber) {
        Map<String, Object> json = new HashMap();
        // taskType 1 搬运
        json.put("taskType", taskType);
        json.put("sendRow", endPoint);
        json.put("pickRow", startPoint);
        json.put("taskNo", currTaskId);
        json.put("ideNumber", currTaskId);
        json.put("processNumber", processNumber);
        String taskJson = JSON.toJSONString(json);
        String result = null;
        try {
            if (vehicleId == 1) {
                result = HttpClient.httpPost("http://192.168.6.51:8009/rgv1/sendTask", taskJson);
            }
            if (vehicleId == 2) {
                result = HttpClient.httpPost("http://192.168.6.51:8009/rgv2/sendTask", taskJson);
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        JSONObject jsonObject = JSON.parseObject(result);
        return Integer.parseInt(String.valueOf(jsonObject.get("msg")));
    }
}
