package com.mt.wms.empty.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.mt.wms.core.base.BaseService;
import com.mt.wms.core.dal.entity.*;
import com.mt.wms.core.dal.service.CurrTaskDetServiceBiz;
import com.mt.wms.core.dal.service.InStockInfoServiceBiz;
import com.mt.wms.core.dal.service.LocationServiceBiz;
import com.mt.wms.core.dal.service.OrderInfoServiceBiz;
import com.mt.wms.core.params.IdParam;
import com.mt.wms.core.vo.R;
import com.mt.wms.empty.params.StockInfoQueryParam;
import com.mt.wms.empty.service.StockInfoService;
import com.mt.wms.empty.vo.CurrTaskDetVo;
import com.mt.wms.empty.vo.StockInfoVo;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * @Author: liguanghao
 * @Date: 2021/12/17 10:24
 * @Version 1.0
 */
@Service
@Transactional
public class StockInfoServiceImpl extends BaseService implements StockInfoService {
    @Resource
    private InStockInfoServiceBiz inStockInfoServiceBiz;
    @Resource
    private LocationServiceBiz locationServiceBiz;
    @Resource
    private CurrTaskDetServiceBiz currTaskDetServiceBiz;
    @Resource
    private OrderInfoServiceBiz orderInfoServiceBiz;
    @Override
    public R<List<CurrTaskDetVo>> get(IdParam idParam) {
        Long taskId = inStockInfoServiceBiz.getOne(new QueryWrapper<InStockInfo>().eq(InStockInfo.LOCATION_ID,idParam.getId())).getTaskId();
        List<CurrTaskDet> currTaskDetList = currTaskDetServiceBiz.list(new QueryWrapper<CurrTaskDet>().eq("curr_task_id", taskId));
        List<CurrTaskDetVo> currTaskDetVoList = com.mt.wms.core.utils.BeanUtils.copyList(currTaskDetList, CurrTaskDetVo.class);
        for (CurrTaskDetVo currTaskDetVo:currTaskDetVoList
             ) {
            String costomerName = orderInfoServiceBiz.getOne(new QueryWrapper<OrderInfo>().eq(OrderInfo.IDEN_CARD_NUM, currTaskDetVo.getIdenCardNum())).getCustomerName();
            currTaskDetVo.setCustomer(costomerName);
        }
        return successful(currTaskDetVoList);
    }

    @Override
    public R<List<StockInfoVo>> list() {
        //获取全部库位
        List<Location> locationList = locationServiceBiz.list();
        List<StockInfoVo> stockInfoVoList=new ArrayList<>();
        for (Location location:locationList
             ) {
            StockInfoVo stockInfoVo=StockInfoVo.builder().build();
            //库位为空
            if (location.getStatus()!=1){
                stockInfoVo.setEmpty(1);
                stockInfoVo.setLocationId(location.getId());
                stockInfoVo.setLocationName(location.getLocationNameAlias());
            }else {
                InStockInfo inStockInfo = inStockInfoServiceBiz.getOne(new QueryWrapper<InStockInfo>()
                        .eq("location_id", location.getId()));
                BeanUtils.copyProperties(inStockInfo,stockInfoVo);
                stockInfoVo.setEmpty(0);
            }
            stockInfoVoList.add(stockInfoVo);
        }
        return successful(stockInfoVoList);
    }

    @Override
    public R<String> top(IdParam idParam) {
        InStockInfo inStockInfo = inStockInfoServiceBiz.getById(idParam.getId());
        Long kilnId = inStockInfo.getKilnId();
        List<InStockInfo> inStockInfoList = inStockInfoServiceBiz.list(new QueryWrapper<InStockInfo>()
                .eq(InStockInfo.KILN_ID, kilnId)
                .eq(InStockInfo.TYPE,1)
                .eq(InStockInfo.STATUS,2)
                .orderByAsc(InStockInfo.IN_TIME));

        if (inStockInfoList.size()<2){
            return R.failed("当前加工炉等待队列只有一个任务，无需置顶");
        }
        LocalDateTime inTime=null;
        inTime = inStockInfoList.get(0).getInTime();
        inStockInfo.setInTime(inTime.minusHours(1));
        inStockInfoServiceBiz.updateById(inStockInfo);
        return R.ok("当前任务置顶成功","");
    }
}
