package com.mt.wms.gateway.config;

import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import springfox.documentation.builders.ApiInfoBuilder;
import springfox.documentation.builders.PathSelectors;
import springfox.documentation.builders.RequestHandlerSelectors;
import springfox.documentation.service.ApiInfo;
import springfox.documentation.service.Contact;
import springfox.documentation.spi.DocumentationType;
import springfox.documentation.spring.web.plugins.Docket;
import springfox.documentation.swagger2.annotations.EnableSwagger2;

/**
 * 服务网关swagger配置
 *
 * @author jiff
 * @date 2018/11/1
 * @since 1.0
 */
@Configuration
@EnableSwagger2
public class SwaggerConfig {
    @Bean
    public Docket passportApi() {
        return new Docket(DocumentationType.SWAGGER_2).groupName("wms-passport")
                .apiInfo(apiInfo("passport", "统一认证接口"))
                .select()
                .apis(RequestHandlerSelectors.basePackage("com.mt.wms.passport"))
                .paths(PathSelectors.any())
                .build();
    }

    @Bean
    public Docket commonApi() {
        return new Docket(DocumentationType.SWAGGER_2).groupName("wms-common")
                .apiInfo(apiInfo("common", "公共接口"))
                .select()
                .apis(RequestHandlerSelectors.basePackage("com.mt.wms.common"))
                .paths(PathSelectors.any())
                .build();
    }

    @Bean
    public Docket upmsApi() {
        return new Docket(DocumentationType.SWAGGER_2).groupName("wms-upms")
                .apiInfo(apiInfo("upms", "通用权限接口"))
                .select()
                .apis(RequestHandlerSelectors.basePackage("com.mt.wms.upms"))
                .paths(PathSelectors.any())
                .build();
    }
    @Bean
    public Docket basicApi() {
        return new Docket(DocumentationType.SWAGGER_2).groupName("wms-basic")
                .apiInfo(apiInfo("basic", "基础信息接口"))
                .select()
                .apis(RequestHandlerSelectors.basePackage("com.mt.wms.basic"))
                .paths(PathSelectors.any())
                .build();
    }
    @Bean
    public Docket emptyApi() {
        return new Docket(DocumentationType.SWAGGER_2).groupName("wms-empty")
                .apiInfo(apiInfo("empty", "订单相关接口"))
                .select()
                .apis(RequestHandlerSelectors.basePackage("com.mt.wms.empty"))
                .paths(PathSelectors.any())
                .build();
    }

    private ApiInfo apiInfo(String title, String description) {// 创建API的基本信息，这些信息会在Swagger UI中进行显示
        return new ApiInfoBuilder()
                .title(title)
                .description(description)
                .termsOfServiceUrl("http://www.mt.com")
                .contact(new Contact("jiff", "", "jiff@ulabcare.com"))
                .version("1.0")
                .build();
    }
}
